SystemOrganization addCategory: #'ModCluster-Management-Core'!
SystemOrganization addCategory: #'ModCluster-Management-Pharo'!
SystemOrganization addCategory: #'ModCluster-Management-Test'!
SystemOrganization addCategory: #'ModCluster-Advertise-Core'!
SystemOrganization addCategory: #'ModCluster-Advertise-Pharo'!
SystemOrganization addCategory: #'ModCluster-Advertise-Test'!

TestCase subclass: #JBAdvertiseParserTest
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Test'!

!JBAdvertiseParserTest methodsFor: 'private' stamp: 'pmm 9/5/2010 20:03'!
sampleMessage
	^'HTTP/1.0 200 OK', String crlf,
	'Date: Sat, 15 Aug 2009 19:15:34 GMT', String crlf,
	'Sequence: 19', String crlf,
	'Digest: 79e92a311922f736ee7c6380e06b18f7', String crlf,
	'Server: aaad5664-b31e-44f8-bab5-53c7f7ea6d62', String crlf,
	'X-Manager-Address: 192.168.1.33:6666', String crlf,
	'X-Manager-Url: /aaad5664-b31e-44f8-bab5-53c7f7ea6d62', String crlf,
	'X-Manager-Protocol: http', String crlf,
	'X-Manager-Host: 127.0.0.1', String crlf, String crlf! !

!JBAdvertiseParserTest methodsFor: 'running' stamp: 'pmm 8/16/2009 12:27'!
setUp
	parser := JBAdvertiseParser new! !

!JBAdvertiseParserTest methodsFor: 'testing' stamp: 'pmm 9/6/2010 12:26'!
testParse
	| advertisement |
	advertisement := parser parse: self sampleMessage.
	self assert: advertisement sequence = '19'.
	self assert: advertisement digest = '79e92a311922f736ee7c6380e06b18f7'.
	self assert: advertisement server = 'aaad5664-b31e-44f8-bab5-53c7f7ea6d62'.
	self assert: advertisement managerAddress = '192.168.1.33:6666'.
	self assert: advertisement managerAddressHost = '192.168.1.33'.
	self assert: advertisement managerAddressPort = 6666.
	self assert: advertisement managerUrl = '/aaad5664-b31e-44f8-bab5-53c7f7ea6d62'.
	self assert: advertisement managerProtocol = 'http'.
	self assert: advertisement managerHost = '127.0.0.1'.
	! !

TestCase subclass: #JBAnnoucingHandlerTest
	instanceVariableNames: 'handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Test'!

!JBAnnoucingHandlerTest methodsFor: 'running' stamp: 'pmm 9/6/2010 18:43'!
setUp
	handler := JBAnnoucingHandler new! !

!JBAnnoucingHandlerTest methodsFor: 'testing' stamp: 'pmm 9/6/2010 18:52'!
testEntryPointsOfDefaultName
	| rootDispachter childDispatcher entryPoints |
	rootDispachter := WADispatcher new.
	rootDispachter register: WAFileHandler new at: 'zork'.
	rootDispachter defaultName: 'zork'.
	entryPoints := handler entryPointsOf: rootDispachter.
	self assert: entryPoints asSet = #('/' '/zork') asSet! !

!JBAnnoucingHandlerTest methodsFor: 'testing' stamp: 'pmm 9/6/2010 18:51'!
testEntryPointsOfNestedDispatcher
	| rootDispachter childDispatcher entryPoints |
	rootDispachter := WADispatcher new.
	rootDispachter register: WAFileHandler new at: 'zork'.
	childDispatcher := WADispatcher new.
	childDispatcher register: WAFileHandler new at: 'bar'.
	rootDispachter register: childDispatcher at: 'foo'.
	entryPoints := handler entryPointsOf: rootDispachter.
	self assert: entryPoints asSet = #('/zork' '/foo/bar') asSet! !

!JBAnnoucingHandlerTest methodsFor: 'testing' stamp: 'pmm 9/6/2010 18:45'!
testEntryPointsOfSimpleRequestHandler
	| entryPoints |
	entryPoints := handler entryPointsOf: WAFileHandler new.
	self assert: entryPoints = #('/')! !

TestCase subclass: #JBDisableApplicationRequestTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Test'!

!JBDisableApplicationRequestTest methodsFor: 'testing' stamp: 'pmm 9/6/2010 12:23'!
testRequestString
	| request |
	request := JBDisableApplicationRequest
		jvmRoute: 'node1'
		aliases: Array new
		contexts: #('myapp' 'ourapp').
	self assert: request requestString = (
		'DISABLE-APP / HTTP/1.0', String crlf,
		'User-Agent: ClusterListener/1.0', String crlf,
		'Connection: Keep-Alive', String crlf,
		'Content-length: 43', String crlf,
		String crlf,
		'JVMRoute=node1&Context=myapp&Context=ourapp')! !

TestCase subclass: #JBDumpRequestTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Test'!

!JBDumpRequestTest methodsFor: 'testing' stamp: 'pmm 9/6/2010 12:24'!
testRequestString
	| request |
	request := JBDumpRequest new.
	self assert: request requestString = (
		'DUMP / HTTP/1.0', String crlf,
		'User-Agent: ClusterListener/1.0', String crlf,
		'Connection: Keep-Alive', String crlf,
		'Content-length: 0', String crlf,
		String crlf,
		'')! !

TestCase subclass: #JBGenericResponseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Test'!

!JBGenericResponseTest methodsFor: 'testing' stamp: 'pmm 9/6/2010 11:17'!
testFromHeader
	| response |
	response := JBGenericResponse fromString: (
		'HTTP/1.1 200 OK', String crlf,
		'Date: Sun, 05 Sep 2010 17:01:54 GMT', String crlf,
		'Server: Apache/2.2.15 (Unix) mod_cluster/1.1.x mod_ssl/2.2.15 OpenSSL/0.9.8k DAV/2 mod_jk/1.2.30', String crlf,
		'Content-Length: 0', String crlf,
		'Keep-Alive: timeout=300', String crlf,
		'Connection: Keep-Alive', String crlf,
		'Content-Type: text/plain').
	self assert: response protocol = 'HTTP/1.1 200 OK'.
	self assert: response contentLength isZero! !

!JBGenericResponseTest methodsFor: 'testing' stamp: 'pmm 9/6/2010 11:19'!
testFromString
	| response |
	response := JBGenericResponse fromString: (
		'HTTP/1.1 200 OK', String crlf,
		'Date: Tue, 25 Aug 2009 20:14:14 GMT', String crlf,
		'Server: Apache/2.2.11 (Unix) mod_ssl/2.2.11 OpenSSL/0.9.8k DAV/2 mod_jk/1.2.27', String crlf,
		'Content-length: 0', String crlf,
		'Connection: close', String crlf,
		'Content-Type: text/plain', String crlf,
		String crlf,
		'').
	self assert: response protocol = 'HTTP/1.1 200 OK'.
	self assert: response headers = (GRSmallDictionary new
		add: 'Date' -> 'Tue, 25 Aug 2009 20:14:14 GMT';
		add: 'Server' -> 'Apache/2.2.11 (Unix) mod_ssl/2.2.11 OpenSSL/0.9.8k DAV/2 mod_jk/1.2.27';
		add: 'Content-length' -> '0';
		add: 'Connection' -> 'close';
		add: 'Content-Type' -> 'text/plain';
		yourself).
	self assert: response body = ''! !

TestCase subclass: #JBInfoRequestTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Test'!

!JBInfoRequestTest methodsFor: 'testing' stamp: 'pmm 9/6/2010 12:24'!
testRequestString
	| request |
	request := JBInfoRequest new.
	self assert: request requestString = (
		'INFO / HTTP/1.0', String crlf,
		'User-Agent: ClusterListener/1.0', String crlf,
		'Connection: Keep-Alive', String crlf,
		'Content-length: 0', String crlf,
		String crlf,
		'')! !

WAError subclass: #JBError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

JBError subclass: #JBInvalidArgumentError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

Object subclass: #JBTodo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBTodo commentStamp: '<historical>' prior: 0!
JBApplicationRequest >> #generateBody should probably user server codec!

WAObject subclass: #JBAdvertiseHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBAdvertiseHandler commentStamp: 'pmm 8/16/2009 11:31' prior: 0!
A JBAdvertiseHandler handles advertisements. The interface in intentially kept polymorphic to blocks.
!

!JBAdvertiseHandler methodsFor: 'public' stamp: 'pmm 8/16/2009 11:31'!
value: anAdvertisement
	self subclassResponsibility! !

JBAdvertiseHandler subclass: #JBAnnoucingHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBAnnoucingHandler methodsFor: 'private' stamp: 'pmm 9/6/2010 18:50'!
addChildrenOf: aDispatcher withPrefix: aString to: aStream
	aDispatcher defaultName isEmptyOrNil
		ifFalse: [ aStream nextPut: aString, '/' ].
	aDispatcher handlers keysAndValuesDo: [ :name :child |
		child isDispatcher
			ifFalse: [ aStream nextPut: aString, '/', name ]
			ifTrue: [ self addChildrenOf: child withPrefix: aString, '/', name to: aStream ] ]! !

!JBAnnoucingHandler methodsFor: 'private' stamp: 'pmm 9/6/2010 16:55'!
aliasOf: aRequestHandler
	^ aRequestHandler serverHostname! !

!JBAnnoucingHandler methodsFor: 'private' stamp: 'pmm 9/6/2010 16:42'!
apatorsAndTypesDo: aTwoArgumentBlock
	WAServerManager default adaptors do: [ :each |
		each isRunning ifTrue: [
			| type |
			type := self typeOf: each.
			type isNil ifFalse: [
				aTwoArgumentBlock value: each value: type ] ] ]! !

!JBAnnoucingHandler methodsFor: 'private' stamp: 'pmm 9/6/2010 18:44'!
entryPointsOf: aRequestHandler
	^ aRequestHandler isDispatcher
		ifFalse: [ #('/') ]
		ifTrue: [ Array streamContents: [ :stream |
			self addChildrenOf: aRequestHandler withPrefix: '' to: stream ] ]! !

!JBAnnoucingHandler methodsFor: 'private' stamp: 'pmm 9/6/2010 16:49'!
typeOf: aServerAdaptor
	aServerAdaptor superclass name = #WAComancheAdaptor
		ifTrue: [ ^ 'http' ].
	aServerAdaptor class name = #WASwazooAdaptor
		ifTrue: [ ^ 'http' ].
	aServerAdaptor class name = #WAWebServerAdaptor
		ifTrue: [ ^ aServerAdaptor isSSLEnabled ifTrue: [ 'https' ] ifFalse: [ 'http' ] ].
	aServerAdaptor superclass name = #AJPAdaptor
		ifTrue: [ ^ 'ajp' ].
	aServerAdaptor class name = #WAComancheAJPAdaptor
		ifTrue: [ ^ 'ajp' ].
	^ nil! !

!JBAnnoucingHandler methodsFor: 'public' stamp: 'pmm 9/6/2010 12:28'!
value: anAdvertisement
	| host port |
	host := anAdvertisement managerAddressHost.
	port := anAdvertisement managerAddressPort.! !

JBAdvertiseHandler subclass: #JBTranscriptHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Test'!

!JBTranscriptHandler class methodsFor: 'examples' stamp: 'pmm 8/25/2009 22:57'!
example
	^JBPharoAdvertiseListener handler: self new! !

!JBTranscriptHandler methodsFor: 'public' stamp: 'pmm 8/16/2009 17:11'!
value: anAdvertisement
	Transcript show: anAdvertisement printString, String cr! !

WAObject subclass: #JBAdvertiseListener
	instanceVariableNames: 'handler parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBAdvertiseListener commentStamp: 'pmm 8/16/2009 11:25' prior: 0!
A JBAnnounceListener is the abstract base class that listens for messages from mod_advertise.

Instance Variables
	handler:		<Object>
	parser:			<JBAdvertiseParser>

handler
	- the object that will receive the callbacks when a new advertisement has arrived
	
parser
	- parses the advertisements
!

!JBAdvertiseListener class methodsFor: 'accessing' stamp: 'pmm 8/16/2009 13:58'!
defaultPort
	"the default port to listen for advertisement"
	^23364! !

!JBAdvertiseListener methodsFor: 'private' stamp: 'pmm 8/16/2009 13:51'!
dataArrived: aString
	handler value: (parser parse: aString)! !

!JBAdvertiseListener methodsFor: 'initialize-release' stamp: 'pmm 8/16/2009 11:28'!
initializeWithHandler: aHandler
	super initialize.
	handler := aHandler.
	parser := JBAdvertiseParser new! !

!JBAdvertiseListener methodsFor: 'public' stamp: 'pmm 8/16/2009 11:26'!
start
	self subclassResponsibility! !

!JBAdvertiseListener methodsFor: 'public' stamp: 'pmm 8/16/2009 11:26'!
stop
	self subclassResponsibility! !

JBAdvertiseListener subclass: #JBPharoAdvertiseListener
	instanceVariableNames: 'port socket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Pharo'!

!JBPharoAdvertiseListener class methodsFor: 'instance creation' stamp: 'pmm 8/16/2009 13:58'!
handler: aHandler
	^self handler: aHandler port: self defaultPort! !

!JBPharoAdvertiseListener class methodsFor: 'instance creation' stamp: 'pmm 8/16/2009 12:26'!
handler: aHandler port: anInteger
	^self basicNew initializeWithHandler: aHandler port: anInteger! !

!JBPharoAdvertiseListener methodsFor: 'initialize-release' stamp: 'pmm 8/16/2009 13:50'!
initializeWithHandler: aHandler port: anInteger
	super initializeWithHandler: aHandler.
	port := anInteger! !

!JBPharoAdvertiseListener methodsFor: 'private' stamp: 'pmm 8/16/2009 17:08'!
listenLoop
	| buffer s |
	buffer := String new: 65507. "max UDP packet size"
	s := socket.
	[ s isValid ] whileTrue: [
		| datagramm data read |
		datagramm := [ s receiveUDPDataInto: buffer ]
			on: ConnectionClosed
			do: [ :error | ^self ].
		read := datagramm first.
		read > 0
			ifTrue: [
				data :=  buffer copyFrom: 1 to: read.
				self dataArrived: data ]
			ifFalse: [
				(Delay forSeconds: 1) wait ] ].! !

!JBPharoAdvertiseListener methodsFor: 'public' stamp: 'pmm 8/16/2009 17:02'!
start
	socket := Socket newUDP.
	socket setPort: port.
	[ self listenLoop ] forkAt: Processor highIOPriority! !

!JBPharoAdvertiseListener methodsFor: 'public' stamp: 'pmm 8/16/2009 13:54'!
stop
	| s |
	s := socket.
	s isNil ifFalse: [
		s destroy ].
	socket := nil! !

WAObject subclass: #JBAdvertiseParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBAdvertiseParser methodsFor: 'public' stamp: 'pmm 8/16/2009 13:09'!
addLine: aString to: aDictionary
	| dotIndex |
	dotIndex := aString indexOf: $:.
	dotIndex isZero ifFalse: [
		| name value |
		name := (aString copyFrom: 1 to: dotIndex - 1) asLowercase.
		value := (aString copyFrom: dotIndex + 1 to: aString size) trimBoth.
		aDictionary at: name put: value ].! !

!JBAdvertiseParser methodsFor: 'public' stamp: 'pmm 8/16/2009 13:12'!
parse: aString
	| crlf position oldPosition headers |
	headers := Dictionary new.
	crlf := String crlf.
	position := 1.
	oldPosition := position.
	position := aString indexOfSubCollection: crlf startingAt: position.
	[ position > 0 ] whileTrue: [
		self addLine: (aString copyFrom: oldPosition to: position - 1) to: headers.
		position := position + crlf size.
		oldPosition := position.
		position := aString indexOfSubCollection: crlf startingAt: position.
	].
	^JBAdvertisement headers: headers! !

WAObject subclass: #JBAdvertisement
	instanceVariableNames: 'date sequence digest server managerAddress managerUrl managerProtocol managerHost'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Advertise-Core'!

!JBAdvertisement class methodsFor: 'instance creation' stamp: 'pmm 8/16/2009 13:19'!
headers: aDictionary
	^self basicNew
		initializeWithDate: (aDictionary at: 'date' ifAbsent: [ nil ])
		sequence: (aDictionary at: 'sequence' ifAbsent: [ nil ])
		digest: (aDictionary at: 'digest' ifAbsent: [ nil ])
		server: (aDictionary at: 'server' ifAbsent: [ nil ])
		managerAddress: (aDictionary at: 'x-manager-address' ifAbsent: [ nil ])
		managerUrl: (aDictionary at: 'x-manager-url' ifAbsent: [ nil ])
		managerProtocol: (aDictionary at: 'x-manager-protocol' ifAbsent: [ nil ])
		managerHost: (aDictionary at: 'x-manager-host' ifAbsent: [ nil ])! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:20'!
date
	^date! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:21'!
digest
	^digest! !

!JBAdvertisement methodsFor: 'initialize-release' stamp: 'pmm 8/16/2009 13:16'!
initializeWithDate: aDateString sequence: aSequenceString digest: aDigestString server: aServerString managerAddress: aManagerAddressString managerUrl: aManagerUrlString managerProtocol: aManagerProtocolString managerHost: aManagerHostString
	super initialize.
	date := aDateString.
	sequence := aSequenceString.
	digest := aDigestString.
	server := aServerString.
	managerAddress := aManagerAddressString.
	managerUrl := aManagerUrlString.
	managerProtocol := aManagerProtocolString.
	managerHost := aManagerHostString
	! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:22'!
managerAddress
	^managerAddress! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 9/6/2010 12:25'!
managerAddressHost
	^self managerAddress copyUpTo: $:! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 9/6/2010 12:26'!
managerAddressPort
	^(self managerAddress copyAfter: $:) greaseInteger! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:23'!
managerHost
	^managerHost! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:22'!
managerProtocol
	^managerProtocol! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:22'!
managerUrl
	^managerUrl! !

!JBAdvertisement methodsFor: 'printing' stamp: 'pmm 8/16/2009 17:16'!
printOn: aStream
	date isNil ifFalse: [
		aStream
			nextPutAll: 'Date: ';
			nextPutAll: date;
			cr ].
	sequence isNil ifFalse: [
		aStream
			nextPutAll: 'Sequence: ';
			nextPutAll: sequence;
			cr ].
	digest isNil ifFalse: [
		aStream
			nextPutAll: 'Digest: ';
			nextPutAll: digest;
			cr ].
	server isNil ifFalse: [
		aStream
			nextPutAll: 'Server: ';
			nextPutAll: server;
			cr ].
	managerAddress isNil ifFalse: [
		aStream
			nextPutAll: 'X-Manager-Address: ';
			nextPutAll: managerAddress;
			cr ].
	managerUrl isNil ifFalse: [
		aStream
			nextPutAll: 'X-Manager-Url: ';
			nextPutAll: managerUrl;
			cr ].
	managerProtocol isNil ifFalse: [
		aStream
			nextPutAll: 'X-Manager-Protocol: ';
			nextPutAll: managerProtocol;
			cr ].
	managerHost isNil ifFalse: [
		aStream
			nextPutAll: 'X-Manager-Host: ';
			nextPutAll: managerHost;
			cr ]! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:21'!
sequence
	^sequence! !

!JBAdvertisement methodsFor: 'accessing' stamp: 'pmm 8/16/2009 11:22'!
server
	^server! !

WAObject subclass: #JBBalancerConf
	instanceVariableNames: 'balancer stickySession stickySessionCookie stickySessionPath stickySessionRemove stickySessionForce waitWorker maxattempts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
balancer
	^ balancer! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/29/2009 10:11'!
balancer: aString
	"Name of the balancer"
	aString size > 40
		ifTrue: [ JBInvalidArgumentError signal: 'balancer must not be bigger than 40 characters' ].
	balancer := aString! !

!JBBalancerConf methodsFor: 'initialize-release' stamp: 'pmm 8/23/2009 22:13'!
initialize
	super initialize.
	self balancer: 'mycluster'.
	self stickySession: true.
	self stickySessionCookie: 'JSESSIONID'.
	self stickySessionPath: 'jsessionid'.
	self stickySessionRemove: false.
	self stickySessionForce: true.
	self waitWorker: (Duration seconds: 0).
	self maxattempts: 1! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
maxattempts
	^ maxattempts! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 9/5/2010 19:50'!
maxattempts: anInteger 
	maxattempts := anInteger greaseString! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
stickySession
	^ stickySession! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:05'!
stickySession: aBoolean
	"true: use JVMRoute to stick a request to a node
	false: ignore JVMRoute"
	stickySession := aBoolean ifTrue: [ 'Yes' ] ifFalse: [ 'No' ]! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
stickySessionCookie
	^ stickySessionCookie! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/29/2009 10:10'!
stickySessionCookie: aString
	"Name of the cookie containing the sessionid."
	aString size > 30
		ifTrue: [ JBInvalidArgumentError signal: 'StickySessionCookie must not be bigger than 30 characters' ].
	stickySessionCookie := aString! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
stickySessionForce
	^ stickySessionForce! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:06'!
stickySessionForce: aBoolean
	"true: return an error if the request can't be routed according to JVMRoute
	false: Route it to another node"
	stickySessionForce := aBoolean ifTrue: [ 'Yes' ] ifFalse: [ 'No' ]! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
stickySessionPath
	^ stickySessionPath! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/29/2009 10:11'!
stickySessionPath: aString
	"Name of the parameter containing the sessionid."
	aString size > 30
		ifTrue: [ JBInvalidArgumentError signal: 'StickySessionPath must not be bigger than 30 characters' ].
	stickySessionPath := aString! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
stickySessionRemove
	^ stickySessionRemove! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:10'!
stickySessionRemove: aBoolean
	"true: remove the sessionid (cookie or parameter) when the request can't be routed to the right node.
	false: send it anyway."
	stickySessionRemove := aBoolean ifTrue: [ 'Yes' ] ifFalse: [ 'No' ]! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:02'!
waitWorker
	^ waitWorker! !

!JBBalancerConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 22:12'!
waitWorker: aDuration 
	waitWorker := aDuration asSeconds! !

WAObject subclass: #JBDocumentation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBDocumentation commentStamp: 'pmm 8/16/2009 11:15' prior: 0!
http://www.jboss.org/community/wiki/Mod-ClusterManagementProtocol
http://www.jboss.org/community/wiki/ModClusterDesign
http://www.jboss.org/community/wiki/Mod-ClusterInternals
http://www.jboss.org/mod_cluster/advertise.html

http://www.jboss.org/community/wiki/Mod-Cluster-INFO-RSP
http://www.jboss.org/community/wiki/Mod-Clusternodebalancer

http://www.jboss.org/mod_cluster/native/config.html
http://www.jboss.org/mod_cluster/java/properties.html
!

WAObject subclass: #JBGenericResponse
	instanceVariableNames: 'protocol headers body'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBGenericResponse class methodsFor: 'instance creation' stamp: 'pmm 9/6/2010 11:13'!
fromHeader: aString
	^self basicNew initializeFromHeader: aString! !

!JBGenericResponse class methodsFor: 'instance creation' stamp: 'pmm 8/25/2009 22:22'!
fromString: aString
	^self basicNew initializeFrom: aString! !

!JBGenericResponse methodsFor: 'private' stamp: 'pmm 9/6/2010 11:12'!
addHeaderLine: aString
	protocol isNil
		ifTrue: [ protocol := aString ]
		ifFalse: [
			| colonIndex |
			colonIndex := aString indexOf: $:.
			headers
				at: (aString copyFrom: 1 to: colonIndex - 1)
				put: (aString copyFrom: colonIndex + 2 to: aString size) ]! !

!JBGenericResponse methodsFor: 'accessing' stamp: 'pmm 8/25/2009 22:32'!
body
	^body! !

!JBGenericResponse methodsFor: 'accessing' stamp: 'pmm 9/6/2010 11:37'!
body: aString
	body := aString! !

!JBGenericResponse methodsFor: 'accessing' stamp: 'pmm 9/6/2010 11:14'!
contentLength
	headers
		at: 'Content-Length'
		ifPresent: [ :value | value greaseInteger ].
	^ 0! !

!JBGenericResponse methodsFor: 'accessing' stamp: 'pmm 8/25/2009 22:32'!
headers
	^headers! !

!JBGenericResponse methodsFor: 'initialize-release' stamp: 'pmm 9/6/2010 11:18'!
initializeFrom: aString
	| crlf start previous current |
	super initialize.
	crlf := String crlf.
	start := 1.
	headers := GRSmallDictionary new.
	previous := aString indexOfSubCollection: crlf startingAt: start ifAbsent: [ 0 ].
	current :=  aString indexOfSubCollection: crlf startingAt: previous + 2 ifAbsent: [ 0 ].
	[ current > (previous + 2) ] whileTrue: [
		self addHeaderLine: (aString copyFrom: start to: previous - 1).
		start := previous + 2.
		previous := current.
		current := aString indexOfSubCollection: crlf startingAt: previous + 2 ifAbsent: [ 0 ].
	].
	self addHeaderLine: (aString copyFrom: start to: previous - 1).
	body := aString copyFrom: current + 2 to: aString size.! !

!JBGenericResponse methodsFor: 'initialize-release' stamp: 'pmm 9/6/2010 11:11'!
initializeFromHeader: aString
	| crlf start previous current |
	super initialize.
	crlf := String crlf.
	start := 1.
	headers := GRSmallDictionary new.
	previous := aString indexOfSubCollection: crlf startingAt: start ifAbsent: [ 0 ].
	current :=  aString indexOfSubCollection: crlf startingAt: previous + 2 ifAbsent: [ 0 ].
	[ current > (previous + 2) ] whileTrue: [
		self addHeaderLine: (aString copyFrom: start to: previous - 1).
		start := previous + 2.
		previous := current.
		current := aString indexOfSubCollection: crlf startingAt: previous + 2 ifAbsent: [ 0 ].
	].
	self addHeaderLine: (aString copyFrom: start to: previous - 1)! !

!JBGenericResponse methodsFor: 'accessing' stamp: 'pmm 8/25/2009 22:51'!
protocol
	^protocol! !

WAObject subclass: #JBNodeConf
	instanceVariableNames: 'jvmRoute domain host port type flushpackets flushwait ping smax ttl timeout'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:28'!
domain
	^ domain! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/29/2009 10:11'!
domain: aString
	aString size > 20
		ifTrue: [ JBInvalidArgumentError signal: 'Domain must not be bigger than 20 characters' ].
	domain := aString! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
flushpackets
	^ flushpackets! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
flushpackets: aString
	flushpackets := aString! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:30'!
flushpacketsAuto
	"wait for flushwait time before sending"
	self flushpackets: 'Auto'! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:30'!
flushpacketsOff
	"don't flush"
	self flushpackets: 'Off'! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:30'!
flushpacketsOn
	"Send immediately"
	self flushpackets: 'On'! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
flushwait
	^ flushwait! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:36'!
flushwait: aDuration
	"Time to wait before flushing."
	flushwait := aDuration asMilliSeconds! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
host
	^ host! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
host: anObject
	host := anObject! !

!JBNodeConf methodsFor: 'initialize-release' stamp: 'pmm 8/23/2009 21:45'!
initializeWithJvmRoute: aString
	super initialize.
	"set all the defaults"
	self jvmRoute: aString.
	self domain: ''.
	self host: 'localhost'.
	self port: 8009.
	self typeAjp.
	self flushpacketsOff.
	self flushwait: (Duration milliSeconds: 10).
	self ping: (Duration seconds: 10).
	self flag: #todo. "smax"
	self ttl: (Duration seconds: 60)! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
jvmRoute
	^ jvmRoute! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/29/2009 10:11'!
jvmRoute: aString
	aString size > 80
		ifTrue: [ JBInvalidArgumentError signal: 'JvmRoute must not be bigger than 80 characters' ].
	jvmRoute := aString! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
ping
	^ ping! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:37'!
ping: aDuration
	"Time to wait for a pong answer to a ping. 0 means we don't try to ping before sending."
	ping := aDuration asSeconds! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
port
	^ port! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
port: anObject
	port := anObject! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
smax
	^ smax! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
smax: anObject
	smax := anObject! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
timeout
	^ timeout! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:39'!
timeout: aDuration
	"Max time httpd will wait for the backend connection."
	timeout := aDuration asSeconds! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
ttl
	^ ttl! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:38'!
ttl: aDuration
	"max time in seconds to life for connection above smax"
	ttl := aDuration asSeconds! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:29'!
type
	^ type! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:40'!
type: aString
	type := aString! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:40'!
typeAjp
	self type: 'ajp'! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:41'!
typeHttp
	self type: 'http'! !

!JBNodeConf methodsFor: 'accessing' stamp: 'pmm 8/23/2009 21:41'!
typeHttps
	self type: 'https'! !

WAObject subclass: #JBRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

JBRequest subclass: #JBApplicationRequest
	instanceVariableNames: 'jvmRoute aliases contexts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBApplicationRequest class methodsFor: 'instance creation' stamp: 'pmm 8/16/2009 21:02'!
jvmRoute: aJvmRouteString aliases: aCollectionAliases contexts: aCollectionOfContexts
	^self basicNew
		initializeWithJvmRoute: aJvmRouteString
		aliases: aCollectionAliases
		contexts: aCollectionOfContexts! !

!JBApplicationRequest methodsFor: 'private' stamp: 'pmm 9/5/2010 20:26'!
generateBody
	| url codec urlString bodySize |
	url := WAUrl new.
	jvmRoute isEmptyOrNil
		ifFalse: [ url addField: 'JVMRoute' value: jvmRoute ].
	aliases do: [ :each |
		url addField: 'Alias' value: each ].
	contexts do: [ :each |
		url addField: 'Context' value: each ].
	codec := GRCodec forEncoding: 'utf-8'.
	urlString := String streamContents: [ :stream |
		| document |
		document := WAUnescapedDocument 
			on: stream
			codec: codec.
		url encodeOn: document ].
	
	"urlString will start with /? skip it"
	^ urlString copyFrom: 3 to: urlString size! !

!JBApplicationRequest methodsFor: 'initialize-release' stamp: 'pmm 8/16/2009 21:00'!
initializeWithJvmRoute: aJvmRouteString aliases: aCollectionAliases contexts: aCollectionOfContexts
	super initialize.
	jvmRoute := aJvmRouteString.
	aliases := aCollectionAliases.
	contexts := aCollectionOfContexts! !

!JBApplicationRequest methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:11'!
name
	self subclassResponsibility! !

JBApplicationRequest subclass: #JBDisableApplicationRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBDisableApplicationRequest methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:10'!
name
	^'DISABLE-APP'! !

JBApplicationRequest subclass: #JBEnableApplicationRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBEnableApplicationRequest methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:10'!
name
	^'ENABLE-APP'! !

JBApplicationRequest subclass: #JBRemoveApplicationRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBRemoveApplicationRequest methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:11'!
name
	^'REMOVE-APP'! !

JBApplicationRequest subclass: #JBStopApplicationRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBStopApplicationRequest methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:10'!
name
	^'STOP-APP'! !

JBRequest subclass: #JBConfigRequest
	instanceVariableNames: 'balancer balanacerConf jvmRoute domain host port type configuration context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBConfigRequest commentStamp: 'pmm 8/15/2009 18:21' prior: 0!
Send configuration information for a node or set of nodes.
http://www.jboss.org/community/wiki/Mod-Clusternodebalancer

Instance Variables
	balanacerConf:		<Object>
	balancer:		<Object>
	configuration:		<Object>
	context:		<Object>
	domain:		<Object>
	host:		<Object>
	jvmRoute:		<Object>
	port:		<Object>
	type:		<Object>

balanacerConf
	- xxxxx

balancer
	- xxxxx

configuration
	- xxxxx

context
	- xxxxx

domain
	- xxxxx

host
	- xxxxx

jvmRoute
	- xxxxx

port
	- xxxxx

type
	- xxxxx
!

JBRequest subclass: #JBDumpRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBDumpRequest methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:13'!
name
	^'DUMP'! !

JBRequest subclass: #JBInfoRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBInfoRequest methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:13'!
name
	^'INFO'! !

!JBRequest methodsFor: 'private' stamp: 'pmm 9/5/2010 20:23'!
generateBody
	"Default implementation takes no arguments.
	Subclasses may want to override."
	^ ''! !

!JBRequest methodsFor: 'accessing' stamp: 'pmm 9/6/2010 11:42'!
requestString
	^String streamContents: [ :stream |
		self requestStringOn: stream ]! !

!JBRequest methodsFor: 'accessing' stamp: 'pmm 9/6/2010 12:20'!
requestStringOn: aStream
	| body |
	aStream
		nextPutAll: self name;
		nextPutAll: ' / HTTP/1.0';
		crlf;
		nextPutAll: 'User-Agent: ClusterListener/1.0';
		crlf;
		nextPutAll: 'Connection: Keep-Alive';
		crlf.
	body := self generateBody.
	aStream
		nextPutAll: 'Content-length: ';
		print: body size;
		crlf;
		crlf.
	aStream nextPutAll: body! !

JBRequest subclass: #JBStatusRequest
	instanceVariableNames: 'jvmRoute load'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBStatusRequest commentStamp: 'pmm 8/15/2009 18:20' prior: 0!
A JBStatusRequest is xxxxxxxxx.

Instance Variables
	jvmRoute:		<String>
	load:		<String>

jvmRoute
	- is what is after the . in the JSESSIONID cookie or the parameter jsessionid. (max size: 80)

load
	- That is a number (string) It is a number between 1 and 100 and defines the normalized weighted load applied to the worker. Only possible values are real load fator. The value "0" means the node is on standby. "-1" means the node is broken (The cluster can't connect to it).
!

!JBStatusRequest methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:18'!
name
	^'STATUS'! !

WAObject subclass: #JBRequestLoop
	instanceVariableNames: 'host port mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

JBRequestLoop subclass: #JBPharoRequestLoop
	instanceVariableNames: 'socket stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Pharo'!

!JBPharoRequestLoop class methodsFor: 'example' stamp: 'pmm 8/25/2009 23:16'!
example
	| loop |
	loop := self new.
	^loop
		sendRequest: JBInfoRequest new
		toHost: '192.168.1.33'
		port: 6666! !

!JBPharoRequestLoop methodsFor: 'public' stamp: 'pmm 9/6/2010 11:46'!
sendRequest: aRequest
	mutex critical: [
		| response contentLength |
		socket isValid ifFalse: [
			socket destroy.
			socket := Socket newTCP.
			socket connectTo: host port: port.
			stream socketStream on: socket ].
		aRequest requestStringOn: stream.
		stream flush.
		response := JBGenericResponse fromHeader: (stream upToAll: String crlf, String crlf).
		contentLength := response contentLength.
		contentLength isZero ifFalse: [
			response body: (stream next: contentLength) ] ]! !

!JBPharoRequestLoop methodsFor: 'public' stamp: 'pmm 9/6/2010 11:48'!
stop
	mutex critical: [
		socket isNil ifTrue: [ ^self ].
		socket destroy.
		socket := nil.
		stream := nil ]! !

!JBRequestLoop methodsFor: 'public' stamp: 'pmm 9/6/2010 11:46'!
sendRequest: aRequest
	self subclassResponsibility! !

!JBRequestLoop methodsFor: 'public' stamp: 'pmm 9/6/2010 11:47'!
stop
	self subclassResponsibility! !

WAObject subclass: #JBResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

JBResponse subclass: #JBDumpResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBDumpResponse methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:16'!
name
	^'DUMP-RSP'! !

JBResponse subclass: #JBInfoResponse
	instanceVariableNames: 'vhost vport rhost rport'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBInfoResponse methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:16'!
name
	^'INFO-RSP'! !

!JBResponse methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:15'!
name
	self subclassResponsibility! !

JBResponse subclass: #JBStatusResponse
	instanceVariableNames: 'jvmRoute state id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModCluster-Management-Core'!

!JBStatusResponse methodsFor: 'accessing' stamp: 'pmm 8/15/2009 18:17'!
name
	^'STATUS-RSP'! !
